@@ -17,6 +17,10 @@ class Changelog:
 
     def initLogs(self) -> None:
         """Fill the changelog."""
+        self.logs.append(
+            ChangelogEntry(5, 24, 3)
+            .addLine("Finally fixed the pricings.")
+        )
         self.logs.append(
             ChangelogEntry(5, 24, 2)
             .addLine("Added \\x1b\\r as newline, just like claude (smort!)")
