@@ -173,11 +173,38 @@ def play_tts(
 def estimated_tokens(ppt: str, messages: Conversation, functions: list) -> int:
     """Estimate the history tokens."""
     try:
-        req = ""
-        req += ppt
-        req += str(json.dumps(messages.get_messages()))
-        req += str(json.dumps(functions))
-        return __internal_get_tokens(req)
+        total_tokens = 0
+
+        # Count prompt tokens if provided
+        if ppt:
+            total_tokens += __internal_get_tokens(ppt)
+
+        # Count message tokens
+        for msg in messages.get_messages():
+            content = msg.get('content', '')
+            if isinstance(content, str):
+                total_tokens += __internal_get_tokens(content)
+            elif isinstance(content, list):
+                # Handle vision messages
+                for item in content:
+                    if item.get('type') == 'text':
+                        total_tokens += __internal_get_tokens(
+                            item.get('text', '')
+                        )
+                    elif item.get('type') == 'image_url':
+                        # Rough estimate for images (varies by size)
+                        total_tokens += 85  # Low detail estimate
+
+        # Count function/tool tokens (rough estimate)
+        if functions:
+            for func in functions:
+                # Count function definition tokens
+                func_str = str(func.get('function', {}))
+                total_tokens += __internal_get_tokens(func_str)
+
+        # Add overhead for message formatting (~4 tokens per message)
+        total_tokens += len(messages.get_messages()) * 4
+        return total_tokens
     except Exception as e:
         logger = getLogger.getLogger(__name__, baseConf.get("debug", False))
         logger.info("An error has occured while estimating tokens:")
@@ -186,31 +213,17 @@ def estimated_tokens(ppt: str, messages: Conversation, functions: list) -> int:
 
 
 def estimated_cost(
-    input_tokens: int, model: str, output_tokens: int = 4096
+    input_tokens: int, model: str, output_tokens: int = 4096, provider: str = ""
 ) -> float:
-    # ppt = price per token: tuple (input, output)
-    per_k = 1000
-    per_m = 1000 * per_k
-    ppt_table = {
-        '00:gpt-4o': (5 / per_m, 15 / per_m),
-        '01:gpt-3': (0.5 / per_m, 1.5 / per_m),
-        '02:gpt-4-turbo': (10 / per_m, 30 / per_m),
-        '03:gpt-4-32k': (60 / per_m, 120 / per_m),
-        '04:gpt-4-': (30 / per_m, 60 / per_m),
-        '05:open-mistral-7b': (0.25 / per_m, 0.25 / per_m),
-        '06:open-mixtral-8x7b': (0.7 / per_m, 0.7 / per_m),
-        '07:open-mixtral-8x22b': (2 / per_m, 6 / per_m),
-        '08:mistral-small': (1 / per_m, 3 / per_m),
-        '09:mistral-medium': (2.7 / per_m, 8.1 / per_m),
-        '10:mistral-large': (4 / per_m, 12 / per_m),
-    }
-    ppt_indexes_sorted = list(sorted(ppt_table.keys()))
-    for index in ppt_indexes_sorted:
-        model_start = ':'.join(index.split(':')[1:])
-        if model.startswith(model_start):
-            ppt = ppt_table[index]
-            return (input_tokens * ppt[0]) + (output_tokens * ppt[1])
-    return 0
+    """Calculate estimated cost based on token counts."""
+    from .pricing import get_model_pricing
+
+    input_price, output_price = get_model_pricing(model, provider)
+
+    if input_price == 0 and output_price == 0:
+        return 0.0
+
+    return (input_tokens * input_price) + (output_tokens * output_price)
 
 
 def estimated_tokens_and_cost(
@@ -218,10 +231,12 @@ def estimated_tokens_and_cost(
     messages: Conversation,
     functions: list,
     model: str,
-    output_tokens: int = 4096
+    output_tokens: int = 4096,
+    provider: str = ""
 ) -> tuple[int, float]:
+    """Estimate tokens and cost with provider support."""
     input_tokens = estimated_tokens(ppt, messages, functions)
-    cost = estimated_cost(input_tokens, model, output_tokens)
+    cost = estimated_cost(input_tokens, model, output_tokens, provider)
     return (input_tokens, cost)
 
 
